/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2001 Hiroyuki Yamamoto
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __ADDRESS_H__
#define __ADDRESS_H__

#include <glib.h>

#define ADDRESS_OBJECT(obj)		((AddressObject *)obj)
#define ADDRESS_OBJECT_TYPE(obj)	(ADDRESS_OBJECT(obj)->type)
#define ADDRESS_ITEM(obj)		((AddressItem *)obj)
#define ADDRESS_GROUP(obj)		((AddressGroup *)obj)
#define ADDRESS_FOLDER(obj)		((AddressFolder *)obj)

#define ADDRESS_TAG_COMMON	"common_address"
#define ADDRESS_TAG_PERSONAL	"personal_address"

typedef struct _AddressTree	AddressTree;
typedef struct _AddressObject	AddressObject;
typedef struct _AddressItem	AddressItem;
typedef struct _AddressGroup	AddressGroup;
typedef struct _AddressFolder	AddressFolder;

#include "compose.h"

typedef enum
{
	ADDR_ITEM,
	ADDR_GROUP,
	ADDR_FOLDER
} AddressObjectType;

struct _AddressTree
{
	GNode *node;

	AddressObject *common;
	AddressObject *personal;
};

struct _AddressObject
{
	AddressObjectType type;

	GNode *node;
};

struct _AddressItem
{
	AddressObject obj;

	gchar *name;
	gchar *address;
	gchar *remarks;
};

struct _AddressGroup
{
	AddressObject obj;

	gchar *name;

	/* Group contains only Items */
	GList *items;
};

struct _AddressFolder
{
	AddressObject obj;

	gchar *name;

	/* Folder contains Groups and Items */
	GList *items;
};

void address_read_file			(void);
void address_export_to_file		(void);

GNode *address_get_tree			(void);
GList *address_get_folder_list		(void);

AddressItem *address_parse_str		(const gchar	*str);

AddressObject *address_folder_new	(const gchar	*name);
AddressObject *address_group_new	(const gchar	*name);
AddressObject *address_item_new		(const gchar	*name,
					 const gchar	*address,
					 const gchar	*remarks);

gchar *address_get_name			(AddressObject	*obj);
GList *address_get_items		(AddressObject	*obj);

void address_add_object			(AddressObject	*parent,
					 AddressObject	*obj);
void address_delete_object		(AddressObject	*obj);

AddressObject *address_find_object_by_name	(AddressObject	*obj,
						 const gchar	*name);
AddressObject *address_find_item_by_name_and_address
						(AddressObject	*obj,
						 const gchar	*name,
						 const gchar	*address);
gint address_obj_name_compare			(gconstpointer	 a,
						 gconstpointer	 b);
gint address_item_compare			(gconstpointer	 a,
						 gconstpointer	 b);

#endif /* __ADDRESS_H__ */
